@@ -374,6 +374,7 @@ class _click_container(_cb_container):
             (to remove the callback, use `m.cb.remove(cbname)`)
 
         """
+
         if self._method == "pick":
             assert self._m.figure.coll is not None, (
                 "you can only attach pick-callbacks after plotting a dataset!"
@@ -660,15 +661,8 @@ class cb_pick_container(_click_container):
         self._add_pick_callback()
 
     def _default_picker(self, artist, event):
-        ax = self._m.figure.ax
 
-        # set max. distance in pixel-coordinates for picking
-        if self._pick_distance is None:
-            maxdist = np.inf
-        else:
-            maxdist = self._pick_distance
-
-        if (event.inaxes != ax) or not hasattr(self._m, "tree"):
+        if (event.inaxes != self._m.ax) or not hasattr(self._m, "tree"):
             return False, dict(ind=None, dblclick=event.dblclick, button=event.button)
 
         # make sure non-finite coordinates (resulting from projections in
@@ -676,20 +670,10 @@ class cb_pick_container(_click_container):
         if not np.isfinite((event.xdata, event.ydata)).all():
             return False, dict(ind=None, dblclick=event.dblclick, button=event.button)
 
-        # use a cKDTree based picking to speed up picks for large collections
+        # find the closest point to the clicked pixel
         dist, index = self._m.tree.query((event.xdata, event.ydata))
 
-        # do this to make sure that we calculate the distance in the right axes!
-        # (necessary for connected pick-callbacks)
-        # for the axes of the original click, this is equal to (event.x, event.y)
-        p1 = ax.transData.transform((event.xdata, event.ydata))
-
-        p2 = ax.transData.transform(
-            (self._m._props["x0"][index], self._m._props["y0"][index])
-        )
-        pdist = np.sqrt(np.sum((p1 - p2) ** 2))
-
-        if pdist < maxdist:
+        if index is not None:
             return True, dict(
                 ind=index, dblclick=event.dblclick, button=event.button, dist=dist
             )
@@ -713,7 +697,7 @@ class cb_pick_container(_click_container):
                 )
             else:
                 clickdict = dict(
-                    ID=getattr(event, "ID", getattr(event, "ind", None)),
+                    ID=getattr(event, "ID", None),
                     pos=getattr(
                         event, "pos", (event.mouseevent.xdata, event.mouseevent.ydata)
                     ),
@@ -810,8 +794,8 @@ class cb_pick_container(_click_container):
 
             dummymouseevent = SimpleNamespace(
                 inaxes=m.figure.ax,
-                dblclick=event.dblclick,
-                button=event.button,
+                dblclick=event.mouseevent.dblclick,
+                button=event.mouseevent.button,
                 xdata=xdata,
                 ydata=ydata,
                 # x=event.mouseevent.x,
@@ -819,8 +803,8 @@ class cb_pick_container(_click_container):
             )
             dummyevent = SimpleNamespace(
                 artist=obj._artist,
-                dblclick=event.dblclick,
-                button=event.button,
+                dblclick=event.mouseevent.dblclick,
+                button=event.mouseevent.button,
                 # inaxes=m.figure.ax,
                 mouseevent=dummymouseevent,
                 # picker_name=picker_name,
@@ -829,7 +813,10 @@ class cb_pick_container(_click_container):
             pick = obj._picker(obj._artist, dummymouseevent)
 
             if pick[1] is not None:
+                dummyevent.ID = pick[1].get("ID", None)
                 dummyevent.ind = pick[1].get("ind", None)
+                dummyevent.val = pick[1].get("val", None)
+
                 if "dist" in pick[1]:
                     dummyevent.dist = pick[1].get("dist", None)
             else:
@@ -876,8 +863,15 @@ class keypress_container(_cb_container):
             self._event = event
 
             if event.key in self.get.cbs:
-                for name, cb in self.get.cbs[event.key].items():
-                    cb(key=event.key)
+                # do this to allow deleting callbacks with a callback
+                cbs = self.get.cbs[event.key]
+                names = list(cbs)
+                for name in names:
+                    if name in cbs:
+                        cbs[name](key=event.key)
+
+                # for name, cb in self.get.cbs[event.key].items():
+                #     cb(key=event.key)
 
         if self._m is self._m.parent:
             self._cid_keypress_event = self._m.figure.f.canvas.mpl_connect(
