@@ -223,16 +223,18 @@ class _wmts_layer(_WebMap_layer):
             additional kwargs passed to the WebMap service request.
             (e.g. transparent=True, time='2020-02-05', etc.)
         """
-        self._m._set_axes()
+        from . import MapsGrid  # do this here to avoid circular imports!
 
-        print(f"EOmaps: Adding wmts-layer: {self.name}")
-        art = self._m.figure.ax.add_wmts(
-            self._wms, self.name, wmts_kwargs=kwargs, interpolation="spline36"
-        )
-        if layer is None:
-            layer = self._m.layer
+        for m in self._m if isinstance(self._m, MapsGrid) else [self._m]:
 
-        self._m.BM.add_bg_artist(art, layer)
+            print(f"EOmaps: Adding wmts-layer: {self.name}")
+            art = m.figure.ax.add_wmts(
+                self._wms, self.name, wmts_kwargs=kwargs, interpolation="spline36"
+            )
+            if layer is None:
+                layer = m.layer
+
+            m.BM.add_bg_artist(art, layer)
 
 
 class _wms_layer(_WebMap_layer):
@@ -254,16 +256,17 @@ class _wms_layer(_WebMap_layer):
             (e.g. transparent=True, time='2020-02-05', etc.)
         """
         print(f"EOmaps: ... adding wms-layer: {self.name}")
-        self._m._set_axes()
+        from . import MapsGrid  # do this here to avoid circular imports!
 
-        art = self._m.figure.ax.add_wms(
-            self._wms, self.name, wms_kwargs=kwargs, interpolation="spline36"
-        )
+        for m in self._m if isinstance(self._m, MapsGrid) else [self._m]:
+            art = m.figure.ax.add_wms(
+                self._wms, self.name, wms_kwargs=kwargs, interpolation="spline36"
+            )
 
-        if layer is None:
-            layer = self._m.layer
+            if layer is None:
+                layer = m.layer
 
-        self._m.BM.add_bg_artist(art, layer)
+            m.BM.add_bg_artist(art, layer)
 
 
 class _WebServiec_collection(object):
@@ -335,17 +338,70 @@ class _WebServiec_collection(object):
 
 
 class REST_API_services:
-    def __init__(self, m, url, name, service_type="wmts", _params={"f": "pjson"}):
+    def __init__(
+        self, m, url, name, service_type="wmts", layers=None, _params={"f": "pjson"}
+    ):
+        """
+        fetch layers from a Rest API
+
+        Parameters
+        ----------
+        m : eomaps.Maps
+            the parent maps object.
+        url : str
+            the url to the API.
+        name : str
+            The name of the API.
+        service_type : str, optional
+            the service-type to use ("wms" or "wmts"). The default is "wmts".
+        layers : set, optional
+            A set of default layers used for delayed fetching (and autocompletion...)
+            As soon as one of the layers is accessed, the API is fetched and the
+            layers are updated according to the current status of the Server.
+            If None or empty, layers are immediately fetched!
+            The default is None.
+        _params : set, optional
+            additional parameters passed to the API. The default is {"f": "pjson"}.
+
+        """
         self._m = m
         self._REST_url = url
         self._name = name
         self._service_type = service_type
         self._params = _params
+        self._fetched = False
+        if layers is None:
+            layers = set()
+        self._layers = layers
+
+        for i in self._layers:
+            setattr(self, i, "NOT FOUND")
+
+        if len(self._layers) == 0:
+            self._fetch_services()
+
+    def __getattribute__(self, name):
+        # make sure all private properties are directly accessible
+        if name.startswith("_"):
+            return object.__getattribute__(self, name)
+
+        # fetch layers on first attempt to get a non-private attribute
+        if not self._fetched:
+            self._fetch_services()
+
+        return object.__getattribute__(self, name)
+
+    def _fetch_services(self):
+        if self._fetched:
+            return
+        # set _fetched to True immediately to avoid issues in __getattribute__
+        self._fetched = True
 
-    def fetch_services(self):
         print(f"EOmaps: ... fetching services for '{self._name}'")
+
         self._REST_API = _REST_API(self._REST_url, _params=self._params)
 
+        found_folders = set()
         for foldername, services in self._REST_API._structure.items():
             setattr(
                 self,
@@ -357,6 +413,18 @@ class REST_API_services:
                     url=self._REST_url,
                 ),
             )
+            found_folders.add(foldername)
+
+        new_layers = found_folders - self._layers
+        if len(new_layers) > 0:
+            print(f"EOmaps: ... found some new folders: {new_layers}")
+
+        invalid_layers = self._layers - found_folders
+        if len(invalid_layers) > 0:
+            print(f"EOmaps: ... could not find the folders: {invalid_layers}")
+        for i in invalid_layers:
+            delattr(self, i)
+
         print("EOmaps: done!")
 
 
@@ -518,6 +586,9 @@ class _xyz_tile_service:
 
         self._maxzoom = maxzoom
 
+    def _reinit(self, m):
+        return _xyz_tile_service(m, self.url, self._maxzoom)
+
     class TileFactory(GoogleWTS):
         def __init__(self, url, *args, **kwargs):
             super().__init__(*args, **kwargs)
@@ -570,35 +641,42 @@ class _xyz_tile_service:
             Additional kwargs passed to the cartopy-wrapper for
             matplotlib's `imshow`.
         """
-        self._m._set_axes()
-        self.kwargs = dict(interpolation=interpolation, alpha=alpha)
-        self.kwargs.update(kwargs)
+        from . import MapsGrid  # do this here to avoid circular imports!
 
-        if transparent is True:
-            self.desired_tile_form = "RGBA"
+        if isinstance(self._m, MapsGrid):
+            for m in self._m:
+                self._reinit(m).__call__(
+                    layer, transparent, alpha, interpolation, **kwargs
+                )
         else:
-            self.desired_tile_form = "RGB"
+            self.kwargs = dict(interpolation=interpolation, alpha=alpha)
+            self.kwargs.update(kwargs)
 
-        if self._event_attached is None:
-            self._event_attached = self._m.figure.f.canvas.mpl_connect(
-                "draw_event", self.ondraw
+            if transparent is True:
+                self.desired_tile_form = "RGBA"
+            else:
+                self.desired_tile_form = "RGB"
+
+            if self._event_attached is None:
+                self._event_attached = self._m.figure.f.canvas.mpl_connect(
+                    "draw_event", self.ondraw
+                )
+                toolbar = self._m.figure.f.canvas.toolbar
+
+                if toolbar is not None:
+                    toolbar.release_zoom = self.zoom_decorator(toolbar.release_zoom)
+                    toolbar.release_pan = self.zoom_decorator(toolbar.release_pan)
+                    toolbar._update_view = self.update_decorator(toolbar._update_view)
+
+            if layer is None:
+                self._layer = self._m.layer
+            else:
+                self._layer = layer
+
+            self._factory = self.TileFactory(
+                self.url, desired_tile_form=self.desired_tile_form
             )
-            toolbar = self._m.figure.f.canvas.toolbar
-
-            if toolbar is not None:
-                toolbar.release_zoom = self.zoom_decorator(toolbar.release_zoom)
-                toolbar.release_pan = self.zoom_decorator(toolbar.release_pan)
-                toolbar._update_view = self.update_decorator(toolbar._update_view)
-
-        if layer is None:
-            self._layer = self._m.layer
-        else:
-            self._layer = layer
-
-        self._factory = self.TileFactory(
-            self.url, desired_tile_form=self.desired_tile_form
-        )
-        self.redraw()
+            self.redraw()
 
     def redraw(self):
         # get and remember the extent
