@@ -1,6 +1,7 @@
 """a collection of useful helper-functions."""
 from itertools import tee
 import re
+import sys
 
 import numpy as np
 from matplotlib.colors import LinearSegmentedColormap, ListedColormap
@@ -30,10 +31,11 @@ def _sanitize(s):
     # taken from https://stackoverflow.com/a/3303361/9703451
     s = str(s)
     # Remove leading characters until we find a letter or underscore
-    s = re.sub("^[^a-zA-Z_]+", "", s)
-
+    s2 = re.sub("^[^a-zA-Z_]+", "", s)
+    if len(s2) == 0:
+        s2 = _sanitize("layer_" + str(s))
     # replace invalid characters with an underscore
-    s = re.sub("[^0-9a-zA-Z_]", "_", s)
+    s = re.sub("[^0-9a-zA-Z_]", "_", s2)
     return s
 
 
@@ -66,6 +68,101 @@ def cmap_alpha(cmap, alpha, interpolate=False):
     return new_cmap
 
 
+# a simple progressbar
+# taken from https://stackoverflow.com/a/34482761/9703451
+def progressbar(it, prefix="", size=60, file=sys.stdout):
+    count = len(it)
+
+    def show(j):
+        x = int(size * j / count)
+        file.write("\r%s[%s%s] %i/%i\r" % (prefix, "#" * x, "." * (size - x), j, count))
+        file.flush()
+
+    show(0)
+    for i, item in enumerate(it):
+        yield item
+        show(i + 1)
+    file.write("\n")
+    file.flush()
+
+
+class searchtree:
+    def __init__(self, m=None, pick_distance=50):
+        """
+        search for coordinates
+
+        Parameters
+        ----------
+        m : eomaps.Maps, optional
+            the maps-object. The default is None.
+        pick_distance : int, optional
+            used to limit the number of pixels in the search to
+            a rectangle of (pick_distance * estimated radius in plot_crs)
+            The default is 50.
+        """
+        self._m = m
+        self._pick_distance = pick_distance
+
+        if self._m.shape.radius_crs != "out":
+            radius = self._m.set_shape._estimate_radius(self._m, "out", np.max)
+        else:
+            radius = self._m.shape.radius
+        self.d = max(radius) * self._pick_distance
+
+        self._misses = 0
+
+    def query(self, x, k=1, d=None):
+        if d is None:
+            d = self.d
+
+        # select a rectangle around the pick-coordinates
+        # (provides tremendous speedups for very large datasets)
+        mx = np.logical_and(
+            self._m._props["x0"] > (x[0] - d), self._m._props["x0"] < (x[0] + d)
+        )
+        my = np.logical_and(
+            self._m._props["y0"] > (x[1] - d), self._m._props["y0"] < (x[1] + d)
+        )
+        m = np.logical_and(mx, my)
+        # get the indexes of the search-rectangle
+        idx = np.where(m)[0]
+        # evaluate the clicked pixel as the one with the smallest
+        # euclidean distance
+
+        if len(idx) > 0:
+            self._misses = 0
+            i = idx[
+                (
+                    (self._m._props["x0"][m] - x[0]) ** 2
+                    + (self._m._props["y0"][m] - x[1]) ** 2
+                ).argmin()
+            ]
+        else:
+            # show some warning if no points are found within the pick_distance
+            self._misses += 1
+
+            if self._misses < 3:
+                text = "Found no data here..."
+            else:
+                text = "Found no data here...\n Increase pick_distance?"
+
+            self._m.cb.click._cb.annotate(
+                pos=x,
+                permanent=False,
+                text=text,
+                xytext=(0.98, 0.98),
+                textcoords=self._m.figure.f.transFigure,
+                horizontalalignment="right",
+                verticalalignment="top",
+                arrowprops=None,
+                fontsize=7,
+                bbox=dict(ec="r", fc=(1, 0.9, 0.9, 0.5), lw=0.25, boxstyle="round"),
+            )
+
+            i = None
+        return None, i
+
+
 class draggable_axes:
     def __init__(self, m, modifier="alt+d", cb_modifier="control"):
         self.modifier = modifier
@@ -358,7 +455,7 @@ class draggable_axes:
             self._m_picked.figure.set_colorbar_position(b)
 
         self.set_annotations()
-        self.m.BM.update(artists=self._ax_picked + self._annotations)
+        self.m.BM.update(artists=self.all_axes + self._annotations)
 
     def _color_axes(self):
         for ax in self.all_axes:
@@ -449,8 +546,9 @@ class draggable_axes:
                         _ax_picked = cbi
                     break
 
-        if self._m_picked is _m_picked and self._cb_picked == _cb_picked:
-            return
+        if self._m_picked is not None:
+            if self._m_picked is _m_picked and self._cb_picked == _cb_picked:
+                return
 
         self._ax_picked = _ax_picked
         self._m_picked = _m_picked
@@ -467,7 +565,7 @@ class draggable_axes:
 
         self.set_annotations()
         self.m.BM.update(
-            layers=[self.m.layer], artists=self._ax_picked + self._annotations
+            layers=[self.m.layer], artists=self.all_axes + self._annotations
         )
 
     def cb_scroll(self, event):
@@ -522,7 +620,7 @@ class draggable_axes:
             self._m_picked.figure.set_colorbar_position(b)
 
         self._color_axes()
-        self.m.BM.update(artists=self._ax_picked + self._annotations)
+        self.m.BM.update(artists=self.all_axes + self._annotations)
 
     def cb_key_press(self, event):
         if (self.f.canvas.toolbar is not None) and self.f.canvas.toolbar.mode != "":
@@ -696,8 +794,8 @@ class BlitManager:
             The artist to be added.  Will be set to 'animated' (just
             to be safe).  *art* must be in the figure associated with
             the canvas this class is managing.
-        bottom : bool
-            Indicator if the artist should be added on top(False) or bottom(True)
+        layer : bool
+            The layer number
         """
         if art.figure != self.canvas.figure:
             raise RuntimeError
@@ -720,8 +818,8 @@ class BlitManager:
             The artist to be added.  Will be set to 'animated' (just
             to be safe).  *art* must be in the figure associated with
             the canvas this class is managing.
-        bottom : bool
-            Indicator if the artist should be added on top(False) or bottom(True)
+        layer : int
+            The layer number
         """
         if art.figure != self.canvas.figure:
             raise RuntimeError
